﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.Algorithms.DataAnalysis {
  public interface IDistance<in T> : IItem {
    /// <summary>
    /// Calculates a distance measure between two aribitrary Vectors. The distance value d is 
    /// 1.) positive  d(x,y)>=0
    /// 2.) symmetric d(x,y) = d(y,x)
    /// 3.) zero-reflexive d(x,x) =0;
    /// </summary>
    /// <param name="a">an array representing point x</param>
    /// <param name="b">>an array representing point y</param>
    /// <returns>d(x,y)</returns>
    double Get(T a, T b);

    /// <summary>
    /// Calculates the correct kernel measure if it is smaller than threshold, but any value greater then threshold if the correct distance is greater.
    /// This is for performace only
    /// </summary>
    /// <param name="a"></param>
    /// <param name="b"></param>
    /// <param name="threshold"></param>
    /// <returns></returns>
    double Get(T a, T b, double threshold);

    /// <summary>
    /// Returns a comparator wich compares the distances to item. (allows for sorting nearest/farthest neighbours)
    /// </summary>
    /// <param name="item"></param>
    /// <returns></returns>
    IComparer<T> GetDistanceComparer(T item);
  }
}
